==> v0.6.0.zip: elasticsearch/modules/elasticsearch/src/main/java/org/elasticsearch/action/search/type/TransportSearchHelper.java <==
/*
 * Licensed to Elastic Search and Shay Banon under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership. Elastic Search licenses this
 * file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.elasticsearch.action.search.type;

import org.elasticsearch.ElasticSearchIllegalArgumentException;
import org.elasticsearch.ElasticSearchIllegalStateException;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchScrollRequest;
import org.elasticsearch.action.search.SearchType;
import org.elasticsearch.action.search.ShardSearchFailure;
import org.elasticsearch.cluster.routing.ShardRouting;
import org.elasticsearch.search.SearchPhaseResult;
import org.elasticsearch.search.internal.InternalScrollSearchRequest;
import org.elasticsearch.search.internal.InternalSearchRequest;
import org.elasticsearch.util.Base64;
import org.elasticsearch.util.Tuple;
import org.elasticsearch.util.Unicode;

import java.io.IOException;
import java.util.Collection;
import java.util.regex.Pattern;

/**
 * @author kimchy (Shay Banon)
 */
public abstract class TransportSearchHelper {


    private final static Pattern scrollIdPattern;

    static {
        scrollIdPattern = Pattern.compile(";");
    }

    /**
     * Builds the shard failures, and releases the cache (meaning this should only be called once!).
     */
    public static ShardSearchFailure[] buildShardFailures(Collection<ShardSearchFailure> shardFailures, TransportSearchCache searchCache) {
        ShardSearchFailure[] ret;
        if (shardFailures.isEmpty()) {
            ret = ShardSearchFailure.EMPTY_ARRAY;
        } else {
            ret = shardFailures.toArray(ShardSearchFailure.EMPTY_ARRAY);
        }
        searchCache.releaseShardFailures(shardFailures);
        return ret;
    }

    public static InternalSearchRequest internalSearchRequest(ShardRouting shardRouting, SearchRequest request) {
        InternalSearchRequest internalRequest = new InternalSearchRequest(shardRouting, request.source());
        internalRequest.extraSource(request.extraSource());
        internalRequest.scroll(request.scroll());
        internalRequest.timeout(request.timeout());
        internalRequest.types(request.types());
        return internalRequest;
    }

    public static InternalScrollSearchRequest internalScrollSearchRequest(long id, SearchScrollRequest request) {
        InternalScrollSearchRequest internalRequest = new InternalScrollSearchRequest(id);
        internalRequest.scroll(request.scroll());
        return internalRequest;
    }

    public static String buildScrollId(SearchType searchType, Iterable<? extends SearchPhaseResult> searchPhaseResults) {
        if (searchType == SearchType.DFS_QUERY_THEN_FETCH || searchType == SearchType.QUERY_THEN_FETCH) {
            return buildScrollId(ParsedScrollId.QUERY_THEN_FETCH_TYPE, searchPhaseResults);
        } else if (searchType == SearchType.QUERY_AND_FETCH || searchType == SearchType.DFS_QUERY_AND_FETCH) {
            return buildScrollId(ParsedScrollId.QUERY_AND_FETCH_TYPE, searchPhaseResults);
        } else {
            throw new ElasticSearchIllegalStateException();
        }
    }

    public static String buildScrollId(String type, Iterable<? extends SearchPhaseResult> searchPhaseResults) {
        StringBuilder sb = new StringBuilder().append(type).append(';');
        for (SearchPhaseResult searchPhaseResult : searchPhaseResults) {
            sb.append(searchPhaseResult.id()).append(':').append(searchPhaseResult.shardTarget().nodeId()).append(';');
        }
        return Base64.encodeBytes(Unicode.fromStringAsBytes(sb.toString()));
    }

    public static ParsedScrollId parseScrollId(String scrollId) {
        try {
            scrollId = Unicode.fromBytes(Base64.decode(scrollId));
        } catch (IOException e) {
            throw new ElasticSearchIllegalArgumentException("Failed to decode scrollId", e);
        }
        String[] elements = scrollIdPattern.split(scrollId);
        @SuppressWarnings({"unchecked"}) Tuple<String, Long>[] values = new Tuple[elements.length - 1];
        for (int i = 1; i < elements.length; i++) {
            String element = elements[i];
            int index = element.indexOf(':');
            values[i - 1] = new Tuple<String, Long>(element.substring(index + 1), Long.parseLong(element.substring(0, index)));
        }
        return new ParsedScrollId(scrollId, elements[0], values);
    }

    private TransportSearchHelper() {

    }

}

==> v0.7.0.zip: elasticsearch/modules/elasticsearch/src/main/java/org/elasticsearch/action/search/type/TransportSearchHelper.java <==
/*
 * Licensed to Elastic Search and Shay Banon under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership. Elastic Search licenses this
 * file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.elasticsearch.action.search.type;

import org.elasticsearch.ElasticSearchIllegalArgumentException;
import org.elasticsearch.ElasticSearchIllegalStateException;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchScrollRequest;
import org.elasticsearch.action.search.SearchType;
import org.elasticsearch.action.search.ShardSearchFailure;
import org.elasticsearch.cluster.routing.ShardRouting;
import org.elasticsearch.search.SearchPhaseResult;
import org.elasticsearch.search.internal.InternalScrollSearchRequest;
import org.elasticsearch.search.internal.InternalSearchRequest;
import org.elasticsearch.util.Base64;
import org.elasticsearch.util.Tuple;
import org.elasticsearch.util.Unicode;

import java.io.IOException;
import java.util.Collection;
import java.util.regex.Pattern;

/**
 * @author kimchy (Shay Banon)
 */
public abstract class TransportSearchHelper {


    private final static Pattern scrollIdPattern;

    static {
        scrollIdPattern = Pattern.compile(";");
    }

    /**
     * Builds the shard failures, and releases the cache (meaning this should only be called once!).
     */
    public static ShardSearchFailure[] buildShardFailures(Collection<ShardSearchFailure> shardFailures, TransportSearchCache searchCache) {
        ShardSearchFailure[] ret;
        if (shardFailures.isEmpty()) {
            ret = ShardSearchFailure.EMPTY_ARRAY;
        } else {
            ret = shardFailures.toArray(ShardSearchFailure.EMPTY_ARRAY);
        }
        searchCache.releaseShardFailures(shardFailures);
        return ret;
    }

    public static InternalSearchRequest internalSearchRequest(ShardRouting shardRouting, SearchRequest request) {
        InternalSearchRequest internalRequest = new InternalSearchRequest(shardRouting);
        internalRequest.source(request.source(), request.sourceOffset(), request.sourceLength());
        internalRequest.extraSource(request.extraSource(), request.extraSourceOffset(), request.extraSourceLength());
        internalRequest.scroll(request.scroll());
        internalRequest.timeout(request.timeout());
        internalRequest.types(request.types());
        return internalRequest;
    }

    public static InternalScrollSearchRequest internalScrollSearchRequest(long id, SearchScrollRequest request) {
        InternalScrollSearchRequest internalRequest = new InternalScrollSearchRequest(id);
        internalRequest.scroll(request.scroll());
        return internalRequest;
    }

    public static String buildScrollId(SearchType searchType, Iterable<? extends SearchPhaseResult> searchPhaseResults) {
        if (searchType == SearchType.DFS_QUERY_THEN_FETCH || searchType == SearchType.QUERY_THEN_FETCH) {
            return buildScrollId(ParsedScrollId.QUERY_THEN_FETCH_TYPE, searchPhaseResults);
        } else if (searchType == SearchType.QUERY_AND_FETCH || searchType == SearchType.DFS_QUERY_AND_FETCH) {
            return buildScrollId(ParsedScrollId.QUERY_AND_FETCH_TYPE, searchPhaseResults);
        } else {
            throw new ElasticSearchIllegalStateException();
        }
    }

    public static String buildScrollId(String type, Iterable<? extends SearchPhaseResult> searchPhaseResults) {
        StringBuilder sb = new StringBuilder().append(type).append(';');
        for (SearchPhaseResult searchPhaseResult : searchPhaseResults) {
            sb.append(searchPhaseResult.id()).append(':').append(searchPhaseResult.shardTarget().nodeId()).append(';');
        }
        return Base64.encodeBytes(Unicode.fromStringAsBytes(sb.toString()));
    }

    public static ParsedScrollId parseScrollId(String scrollId) {
        try {
            scrollId = Unicode.fromBytes(Base64.decode(scrollId));
        } catch (IOException e) {
            throw new ElasticSearchIllegalArgumentException("Failed to decode scrollId", e);
        }
        String[] elements = scrollIdPattern.split(scrollId);
        @SuppressWarnings({"unchecked"}) Tuple<String, Long>[] values = new Tuple[elements.length - 1];
        for (int i = 1; i < elements.length; i++) {
            String element = elements[i];
            int index = element.indexOf(':');
            values[i - 1] = new Tuple<String, Long>(element.substring(index + 1), Long.parseLong(element.substring(0, index)));
        }
        return new ParsedScrollId(scrollId, elements[0], values);
    }

    private TransportSearchHelper() {

    }

}
